Method: Fast::ExperimentFile#ok_with
- Defined in:
- lib/fast/experiment.rb
#ok_with(combination) ⇒ Object
Keep track of ok experiments depending on the current combination. It keep the combinations unique removing single replacements after the first round.
273 274 275 276 277 278 279 280 |
# File 'lib/fast/experiment.rb', line 273 def ok_with(combination) @ok_experiments << combination return unless combination.is_a?(Array) combination.each do |element| @ok_experiments.delete(element) end end |